# -*- coding: UTF-8 -*-

#   icd_import.py   2020.7.13   kita
#!! 購入品インポートデーターの一括処理 Script

# 必要なファイル
#   maker_code_dir.csv  zen_han.py  icd_import.txt

# 履歴
# 2021.6.3      書き込み線種を実線に変更する
# 2021.10.25    ソリッドの軽量化をマクロで実現。5回繰り返します。
#               ただし、マクロとの相性が悪いようなので、保存後にファイルを閉じることにしました。
# 2022.4.28     パーツ解除のチェックボックス追加
# 2023.6.29     材質設定できないモデルへの対応(try節を利用)
#               設定値を保存、読込


import clr                  # .NETをimport
clr.AddReference('sxnet')   # 
from sxnet import *     # 全てのクラスをimport
SxSys.init(3999)

import tkinter as tk
import tkinter.ttk as ttk       # ComboBoxを使うのに必要
import os, sys, codecs, re, glob
import zen_han     #全角<-->半角変換
import subprocess
import time

color_dic = {}  # 色辞書

color_dic['無指定'] = None
color_dic['白'] = SxSys.COLOR_WHITE
color_dic['赤'] = SxSys.COLOR_RED
color_dic['黄'] = SxSys.COLOR_YELLOW
color_dic['青'] = SxSys.COLOR_BLUE
color_dic['マゼンタ'] = SxSys.COLOR_MAGENTA
color_dic['シアン'] = SxSys.COLOR_CYAN
color_dic['灰'] = SxSys.COLOR_GRAY
color_dic['橙'] = SxSys.COLOR_ORANGE
color_dic['薄緑'] = SxSys.COLOR_LIGHT_GREEN
color_dic['空色'] = SxSys.COLOR_SKY_BLUE
color_dic['紫'] = SxSys.COLOR_PURPLE
color_dic['青緑'] = SxSys.COLOR_BLUE_GREEN
color_dic['薄橙'] = SxSys.COLOR_FRESH
color_dic['黒'] = SxSys.COLOR_BLACK

#num_max = 10    # 最大開きっぱなし数

if len(sys.argv) < 2:   #引き数無し
    #list_data_fld = 'z:/ListData/'
    fn_mk_cd = 'maker_code_dir.csv'
else:
    fn_mk_cd = sys.argv[1]


# maker code 一覧辞書
if os.path.exists(fn_mk_cd):
    mk_cd_dic = {}      # Maker Code 辞書 mk_cd_dic[889] = [NIC,NICｵｰﾄﾃｯｸ,ＮＩＣオートテック,NIC AUTO TEC,NICオートテック]
    fp = codecs.open(fn_mk_cd,"r","utf-8")
    lines = fp.readlines()
    fp.close()
    for line in lines:
        if line.find('#') > -1:   # '#'を含む行はスキップ コメント行除外
            continue
        tmp = re.split(',',line.rstrip())
        mk_cd_dic[tmp[0]] = tmp[1:]  # offset1から末尾までのリスト
    #print(mk_cd_dic)
else:
    print('\n  メーカーコード一覧csv Fileがありません\007\n')


#------------------------------------------------------------------
fn_mtrl = 'C:\\ICADSX\\USER\\ETC\\zaishitu.csv'      #材質設定File
#fn_mtrl = 'C:\\ICADSX\\ETC\\icad_material.csv'      #材質設定File
#------------------------------------------------------------------

fp = open(fn_mtrl,'r')
lines = fp.readlines()
fp.close()
mtrl_dic = {}       # 材質辞書 mtrl['SS400'] = ['00炭素鋼','7.85']
for line in lines:
    tmp = re.split(',',line.strip())
    mtrl_dic[tmp[1]] = [tmp[0], tmp[2]]

# 設定値保存ファイル
tmp_fn = os.path.basename(__file__)
save_fn = tmp_fn.replace('.py','.set')  # Global

# 初期設定

#------------------
#   ヘルプボタン
#------------------
def push_help():
    cmd = 'notepad icd_import.txt'
    result = subprocess.Popen(cmd,shell=True)

def save_data(save_fn):     # 設定値保存
    ss= ''
    ss= ss+ 'target_fld= '+ e_target_fld.get()+'\n' #0  str     ex.C:/zip/
    ss= ss+ 'hnmi= '+ e_hnmi.get()+'\n'         #1  str     ex.薄形シリンダー
    ss= ss+ 'mk_nm= '+ e_mk_nm.get()+'\n'       #2  str     ex.SMC
    ss= ss+ 'mk_cd= '+ e_mk_cd.get()+'\n'       #3  int     ex.145
    ss= ss+ 'ext= '+ v_ext.get()+'\n'           #4  str     ex.stp
    ss= ss+ 'zai= '+ v_zai.get()+'\n'           #5  str     ex.SS400
    ss= ss+ 'col= '+ v_col.get()+'\n'           #6  str     ex.無色,..
    if v_pk.get():
        ss= ss+ 'pk= '+ 'True\n'             #7  bool    True, False パーツ解除
    else:
        ss= ss+ 'pk= '+ 'False\n'             #7  bool    True, False パーツ解除

    #print(ss)   ####

    fp = codecs.open(save_fn,'w', encoding='utf-8')
    fp.write(ss)
    fp.close()

def read_data(read_fn):      # 設定値の読み込み 起動時
    fp = codecs.open(read_fn, 'r', encoding='utf-8')
    lines = fp.readlines()
    #print(lines)    
    fp.close()
    i= 0; #entry_txt= []
    for line in lines:
        line = line.rstrip()    # 行末の改行削除
        if line.find('#') > -1:  # #が含まれてるなら
            continue

        tmp= line.split(' ')
        if len(tmp)== 1:    # 未記入
            continue

        if tmp[0].find('target_fld') > -1:
            e_target_fld.delete(0,tk.END)
            e_target_fld.insert(0,tmp[1])
        elif tmp[0].find('hnmi') > -1:
            e_hnmi.delete(0,tk.END)
            e_hnmi.insert(0,tmp[1])
        elif tmp[0].find('mk_nm') > -1:
            e_mk_nm.delete(0,tk.END)
            e_mk_nm.insert(0,tmp[1])
        elif tmp[0].find('mk_cd') > -1:
            e_mk_cd.delete(0,tk.END)
            e_mk_cd.insert(0,tmp[1])
        elif tmp[0].find('ext') > -1:
            v_ext.set(tmp[1])
        elif tmp[0].find('zai') > -1:
            v_zai.set(tmp[1])
        elif tmp[0].find('col') > -1:
            v_col.set(tmp[1])
        elif tmp[0].find('pk') > -1:
            if tmp[1]== 'True':
                v_pk.set(True)          # Parts解除
            else: 
                v_pk.set(False)         # Parts解除しない

        #entry_txt.append(line) 
        #print(i, line)    #!!!!
        i += 1



def on_closing():       # 終了時の処理
    save_data(save_fn)
    print('\n   %s を終了しました\n' % os.path.basename(__file__))

    root.destroy()



#---------------------------------------
#   Maker Code 検索
#---------------------------------------
def push_check():     # checkボタン maker name -> maker Code 変換
    mk_nm = e_mk_nm.get()
    if mk_nm == '': return
    mk_cd = e_mk_cd.get()
    flg = 0
    for key in mk_cd_dic:
        #print(type(mk_cd_dic[key]))
        #if isinstance(mk_cd_dic[key], list) == True:
        #print(mk_nm, mk_cd_dic[key])
        for tmp in mk_cd_dic[key]:
            #if tmp.find(mk_nm) > -1:
            if tmp == mk_nm:
                e_mk_cd.delete(0,tk.END)  # エントリー中身を削除
                e_mk_cd.insert(0,key)     # エントリーの中身を設定
                #print(key, mk_cd_dic[key])
                flg = 1
                break
        if flg == 1: break

    if flg == 0: 
        e_mk_cd.delete(0,tk.END)  # エントリー中身を削除
        e_mk_cd.insert(0,'non')     # エントリーの中身を設定

        print('メーカーコードを検索できませんでした')
        #elif type(mk_cd_dic[key]) == str: # これ以下は不要かも   
        #    if mk_cd_dic[key].find(mk_nm) > -1:
        #        e_mk_cd.delete(0,tk.END)  # エントリー中身を削除
        #        e_mk_cd.insert(0,key)     # エントリーの中身を設定
        #        break


    # File数チェック
    ext = v_ext.get()           # 拡張子 
    path = e_target_fld.get()   # 処理対象フォルダー
    file_list = glob.glob(path + '*.' + ext)
    num = len(file_list)
    print('\n   処理対象ファイル数は ' + str(num) +' ケです\n')

    #if num > num_max:
    #   print('\n   11個以上のファイルは保存後に閉じます\007\n')



def set_pf_toppart(pf_dic, inf):    # トップパーツ付加情報と材質と色を設定
    #print(pf_dic, inf)
    col = v_col.get()
    wf = SxWF.getActive()
    wf.setNameTopPart(pf_dic['Sys_Parts_Name'], pf_dic['Sys_Parts_Comment'])
    ss0 =''
    for key in pf_dic:
        ss0 = ss0 + key + ',"' + pf_dic[key] + '",'
    
    ss = ss0[:-1]       #  最後の一文字削除
    #print(ss)
    wf.setInfExTopPart(ss)

    seg_list = wf.getTopSegList(0,0,False)     # 自WF内の最上位のセグメントを取得(パーツ以外)
    if seg_list != None:
        for seg in seg_list:
            if col != '無指定':
                # 色設定
                seg.setColor(color_dic[col])
                #print('A')
            # 材質設定
            try:
                #print('B')
                seg.setInfMaterial(inf)
            except:
                print('材質設定できない要素を飛ばします\007')
        
    else:
        parts = wf.getPartList()
        for part in parts:
            if col != '無指定':
                # 色設定
                #part.setColor(SxSys.COLOR_FRESH)
                part.setColor(color_dic[col])
                #print('C')
    
            # 材質設定
            try:
                #print('D')
                part.setInfMaterial(inf)
            except:
                print('材質設定できない要素を飛ばします\007')
                continue


def weight_saving1(cnt):    #これでもOK!　But マクロの配置が必要
    ss = ';MACRO;LET /keiryou/ @GO'
    SxSys.command(ss,True)  # macro mode　MACRO mode


#   軽量化
def weight_saving(cnt):     #OK!
    ss=';PSTRAN '
    ss=ss+';WRK                                                                            '
    ss=ss+';OPT                                                                            '
    ss=ss+'@WIN                                                                            '
    ss=ss+'X -83.167786 Y 79.896927 Z 3.270831 ,                                           '
    ss=ss+'X 170.437363 Y -56.199898 Z -114.237442 ,                                       '
    #ss=ss+'S 1 X -83.167786 Y 79.896927 Z 3.270831 ,                                       '
    #ss=ss+'S 1 X 170.437363 Y -56.199898 Z -114.237442 ,                                   '
    ss=ss+'@GO                                                                             '
    for i in range(cnt):
        try:
            SxSys.command(ss,True)  # MACRO mode
            time.sleep(1)
        except:
            print('\n   軽量化に失敗しました\007')
        #input('Key In')
        #print('軽量化 %d 実行' % i)

def parts_free():
    part_lst = SxWF.getActive().getPartList()   # 全ての3次元パーツを取得
    #指定パーツ群を一括解除
    SxEntPart.free(part_lst)

    
#------------------
#   実行ボタン
#------------------
def push_exe():     # Impot操作一括処理
    # 書き込み線種を実線に
    SxSys.setPenStyle(SxSys.STYLE_SOLID)    # 実線
    SxSys.setPenWidth(SxSys.WIDTH_MIDDLE)   # 中線
    # 対象ファイルリスト作成
    ext = v_ext.get()           # 拡張子 
    path = e_target_fld.get()   # 処理対象フォルダー
    #TF_close = v_close.get()    # 保存後閉じるかどうか True:閉じる False:閉じない
    file_list = glob.glob(path + '*.' + ext)
    
    #print(ext, path, file_list)
    
    pf_dic = {}
    cmnt= e_hnmi.get() 
    pf_dic['Sys_Parts_Comment'] = cmnt
    pf_dic['User_Type'] = '2購入'
    pf_dic['User_WBHNA'] = cmnt
    zai = v_zai.get()
    pf_dic['User_WBZAI1'] = zen_han.Han2Zen(zai)
    pf_dic['User_WCMNA'] = e_mk_nm.get()
    pf_dic['User_WCMCD'] = e_mk_cd.get()
    pf_dic['User_WCRSV1'] = 'Product by Python' 


    # 材質情報設定
    mat_inf = SxSys.getInfMaterials()
    for inf in mat_inf:
        #print(inf.matid, inf.name, inf.spe_grav)
        if inf.matid.find(zai) > -1:
            break
    #mat_inf.matid = zai
    #mat_inf.name = mtrl_dic[zai][0]
    #mat_inf.spe_grav =  mtrl_dic[zai][1]
    
    num = 0     # 処理個数
    for fn in file_list:
        file = SxFileModel(fn)
        try:
            if ext == 'x_b' or ext == 'x_t':
                model = file.openPS(True)          # modelを開く   DataCheck ON   
            else:
                model = file.open(False)          # modelを開く   読み取り専用OFF   
        except:
            print('     %s File名に禁止文字が含まれています' % fn)
            continue

        # Part解除
        if v_pk.get():  # Parts解除ボタン　ONNなら
            parts_free()

        # 軽量化マクロ実行
        if ext != 'icd':
            cnt = 5
            #weight_saving1(cnt)
            weight_saving(cnt)

        model.setComment(cmnt)  # 保存オプションのコメント用
        fn_icd = fn.replace(ext,'icd')
        fn1 = os.path.basename(fn_icd).replace('.icd','')
        path = os.path.dirname(fn_icd)
        pf_dic['Sys_Parts_Name'] = fn1 
        pf_dic['User_WCTYP'] = fn1 
       
        #print('パーツ付加情報設定') ###
        set_pf_toppart(pf_dic, inf)    # トップパーツ付加情報設定 
        
        
        #print(path, fn1, cmnt)
        model.save(path, fn1, cmnt,7,7)      # Model保存
        #model.save(path, fn1, cmnt,0,0)      # Model保存
        num += 1
        #if TF_close == True or num > num_max:   # Check Box True  or  個数が num_max以上なら
        #    SxModel.getActive().delete(False)     # modelを閉じる    
        SxModel.getActive().delete(False)     # modelを閉じる    

    print('\n   全てのファイルのインポートを終了しました\007\n')
    
#def cb_selected_ext(event):
#    print('v_ext = %s' % v_ext.get())

#def cb_selected_zai(event):
#    print('v_zai = %s' % v_zai.get())






#--------------------------------------------------
#   画面設定
#--------------------------------------------------
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title('icd_import.py Ver.1.0')   #タイトル
#root.minsize(600,350)    # Windowの大きさ
# blue  red  khaki3  cyan  white

#--------------------------------------------------
# ボタン等の部品配置
#--------------------------------------------------
btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'
x00 = 10; y00 = 5
dx = 70;  dy = 30
wdth = 16
nx = 3.7; ny = 7
ss_size = '%dx%d+%d+%d' % (x00+dx*nx, y00+dy*ny, 1100, 0)
root.geometry(ss_size)

#-------------------------------------------------------
ss = 'ヘルプ';  nx = 2.6; ny = 0    # Buttn
#-------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_help, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy)



#---------------------------------------------------
ss = 'Target Fld' ; nx = 0; ny = 0  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_target_fld= tk.Entry(root, width=wdth,bg=e_bcol)  # エントリー部品の生成
e_target_fld.place(x=x00+(nx+1)*dx,y=y00+ny*dy)
e_target_fld.insert(0,'c:\zip\\')     # エントリーの中身を設定

#----------------------------------------------------------
ss = '拡  張  子' ; nx = 0; ny = 1  # Label & CoboBox
#----------------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)

v_ext = tk.StringVar()
cb = ttk.Combobox(root, width=wdth-3,textvariable=v_ext) #, style="BW.TLabel")
#cb.bind('<<ComboboxSelected>>', cb_selected_ext)
cb['values']=('x_b','x_t','iges', 'igs','stl','step','stp','STEP')
cb.set('x_b')
cb.place(x=x00+(nx+1)*dx,y=y00+ny*dy)
#cb.grid(column=nx+2, row=ny+1, padx=2, pady=2)



#---------------------------------------------------
ss = '品      名' ; nx = 0; ny = 2  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_hnmi= tk.Entry(root, widt=wdth,bg=e_bcol)  # エントリー部品の生成
e_hnmi.place(x=x00+(nx+1)*dx,y=y00+ny*dy)
e_hnmi.insert(0,'')     # エントリーの中身を設定

#---------------------------------------------------
ss = '材     質' ; nx = 0; ny = 3  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
#e_zai= tk.Entry(root, width=wdth,bg=e_bcol)  # エントリー部品の生成
#e_zai.place(x=x00+(nx+1)*dx,y=y00+ny*dy)


v_zai = tk.StringVar()
cb = ttk.Combobox(root, width=wdth-3,textvariable=v_zai) #, style="BW.TLabel")
#cb.bind('<<ComboboxSelected>>', cb_selected_zai)
cb['values']=('SUS304','SS400', 'S45C','A1000','C3600','NBR','PTFE','PET')
cb.set('SS400')
cb.place(x=x00+(nx+1)*dx,y=y00+ny*dy)

#---------------------------------------------------
ss = 'メーカー名' ; nx = 0; ny = 4  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_mk_nm= tk.Entry(root, width=wdth,bg=e_bcol)  # エントリー部品の生成
e_mk_nm.place(x=x00+(nx+1)*dx,y=y00+ny*dy)
e_mk_nm.insert(0,'')     # エントリーの中身を設定

#---------------------------------------------------
ss = 'MakerCode' ; nx = 0; ny = 5  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_mk_cd= tk.Entry(root, width=wdth,bg=e_bcol)  # エントリー部品の生成
e_mk_cd.place(x=x00+(nx+1)*dx,y=y00+ny*dy)
#e_mk_cd.insert(0,'291')     # エントリーの中身を設定

#-------------------------------------------------------
ss = 'Part解除';  nx = 2.6; ny = 2    # Checkbuttn
#-------------------------------------------------------
v_pk = tk.BooleanVar()   # 論理値型の変数 Instance
v_pk.set(True)          # 初期値 Parts解除
#v_pk.set(False)          # 初期値 Parts解除しない
cb_pk = tk.Checkbutton(root,text=ss,variable=v_pk)
cb_pk.place(x=x00+nx*dx,y=y00+ny*dy)


#-------------------------------------------------------
ss = 'Code検索';  nx = 2.6; ny = 4    # Buttn
#-------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_check, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy)

#-------------------------------------------------------
ss = '実  行';  nx = 2.6; ny = 5    # Buttn
#-------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_exe, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy)


#---------------------------------------------------
ss = ' Color ' ; nx = 0; ny = 6  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)

v_col = tk.StringVar()
cb = ttk.Combobox(root, width=wdth-3,textvariable=v_col) #
#cb.bind('<<ComboboxSelected>>'), cb_selected_col)
cb['values']=('白','赤','黄','青', 'マゼンタ','シアン','灰','橙','薄緑','空色','紫','青緑','薄橙','黒','無指定')
cb.set('薄橙')
cb.place(x=x00+(nx+1)*dx,y=y00+ny*dy)

#------------------------------------------------------ 
#   起動時の処理
#------------------------------------------------------
if os.path.exists(save_fn):      # save_fn='icd_import.set 
    read_data(save_fn)
    print('\n   %s 読込に成功しました\n' % save_fn) 

else:   # save_fnが無いとき
    print('\n   %s が無いので設定値をを初期化します' % save_fn)

root.protocol("WM_DELETE_WINDOW", on_closing)   # 終了時の処理
root.mainloop()
